
#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/graphics.h>
#include <proto/intuition.h>
#include <proto/gadtools.h>

#define GID_UP    1001
#define GID_DOWN  1002

int main (void)

{
struct Screen *scr;
struct Window *win;
struct IntuiMessage *mess;
BOOL cont;
long scroll = 0;
struct Gadget *glist = NULL;
struct Gadget *gad;
struct NewGadget ng;
long winw,winh;
long y = 0;

scr = LockPubScreen (NULL);

gad = CreateContext (&glist);

ng.ng_LeftEdge   = 0;
ng.ng_TopEdge    = 20 * scr->RastPort.TxHeight;
ng.ng_Width      = 20 * scr->RastPort.TxWidth;
ng.ng_Height     = scr->RastPort.TxHeight + 6;
ng.ng_GadgetText = "Up";
ng.ng_TextAttr   = scr->Font;
ng.ng_GadgetID   = GID_UP;
ng.ng_Flags      = 0;
ng.ng_VisualInfo = GetVisualInfo (scr,TAG_END);
gad = CreateGadget (BUTTON_KIND,gad,&ng,GA_Immediate,TRUE,TAG_END);

ng.ng_LeftEdge  += ng.ng_Width;
ng.ng_GadgetText = "Down";
ng.ng_GadgetID   = GID_DOWN;
gad = CreateGadget (BUTTON_KIND,gad,&ng,GA_Immediate,TRUE,TAG_END);

winw = ng.ng_LeftEdge + ng.ng_Width + scr->WBorLeft + scr->WBorRight;
winh = ng.ng_TopEdge + ng.ng_Height + scr->WBorTop + scr->WBorBottom + scr->RastPort.TxHeight + 1;

win = OpenWindowTags (NULL,
		WA_Left,(scr->Width - winw) / 2,
		WA_Top,(scr->Height - winh) / 2,
		WA_Width,winw,
		WA_Height,winh,
		WA_Flags,WFLG_CLOSEGADGET | WFLG_DRAGBAR | WFLG_DEPTHGADGET | WFLG_ACTIVATE | WFLG_GIMMEZEROZERO | WFLG_NOCAREREFRESH,
		WA_IDCMP,IDCMP_CLOSEWINDOW | IDCMP_VANILLAKEY | IDCMP_GADGETUP | IDCMP_GADGETDOWN,
		WA_Gadgets,glist,
		TAG_END);

if (!win)
	{
	Printf ("cannot open window\n");
	return (RETURN_FAIL);
	}
else
	{
	GT_RefreshWindow (win,NULL);

	SetFont (win->RPort,scr->RastPort.Font);
	SetABPenDrMd (win->RPort,1,0,JAM2);

	cont = TRUE;
	do	{
		if (scroll)
			{
			if (scroll > 0)
				{
				if (gad->Flags & GFLG_SELECTED)
					{
					ScrollRaster (win->RPort,0,-1,0,0,win->GZZWidth - 1,win->GZZHeight - ng.ng_Height - 1);
					y ++;
					if (y >= win->RPort->TxHeight)
						{
						y = 0;
						Move (win->RPort,0,win->RPort->TxBaseline);
						Text (win->RPort,"Text Text Text Text Text Text Text Text Text",44);
						}
					}
				}
			else
				{
				if (gad->Flags & GFLG_SELECTED)
					{
					ScrollRaster (win->RPort,0,1,0,0,win->GZZWidth - 1,win->GZZHeight - ng.ng_Height - 1);
					y --;
					if (y < 0)
						{
						y = win->RPort->TxHeight - 1;
						Move (win->RPort,0,win->GZZHeight - ng.ng_Height - win->RPort->TxHeight + win->RPort->TxBaseline);
						Text (win->RPort,"Text Text Text Text Text Text Text Text Text",44);
						}
					}
				}

			WaitTOF();

			if (SetSignal (0,0) & SIGBREAKF_CTRL_C)
				cont = FALSE;
			}
		else
			{
			if (Wait ((1L << win->UserPort->mp_SigBit) | SIGBREAKF_CTRL_C) & SIGBREAKF_CTRL_C)
				cont = FALSE;
			}

		while (mess = GT_GetIMsg (win->UserPort))
			{
			switch (mess->Class)
				{
			case IDCMP_GADGETDOWN:
				gad = (struct Gadget *)mess->IAddress;
				switch (gad->GadgetID)
					{
				case GID_UP:
					scroll = -1;
					break;
				case GID_DOWN:
					scroll = 1;
					break;
					}
				break;
			case IDCMP_GADGETUP:
				scroll = 0;
				break;
			case IDCMP_VANILLAKEY:
				if (mess->Code == 0x1b) /* Esc */
					cont = FALSE;
				break;
			case IDCMP_CLOSEWINDOW:
				cont = FALSE;
				break;
				}
			GT_ReplyIMsg (mess);
			}

		}
	while (cont);

	CloseWindow (win);
	}

FreeGadgets (glist);

FreeVisualInfo (ng.ng_VisualInfo);

UnlockPubScreen (NULL,scr);

return (RETURN_OK);
}

